/*****
 *                                                                   
 *                   Searduino
 *                      
 *   Copyright (C) 2011 Henrik Sandklef 
 *                                                                   
 * This program is free software; you can redistribute it and/or     
 * modify it under the terms of the GNU General Public License       
 * as published by the Free Software Foundation; either version 3    
 * of the License, or any later version.                             
 *                                                                   
 *                                                                   
 * This program is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     
 * GNU General Public License for more details.                      
 *                                                                   
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software       
 * Foundation, Inc., 51 Franklin Street, Boston,            
 * MA  02110-1301, USA.                                              
 ****/

#include "wiring_private.h"
#include "pins_arduino.h"
#include "../include/error.h"
#include "utils/include/print.h"
#include "utils/include/types.h"
#include "utils/include/error.h"

#include "comm.h"


typedef struct arduino_pin
{
  uint8_t mode;
  uint8_t val;
} arduino_pin;

static arduino_pin arduino_out_pins[NR_OF_OUT_PINS];
static arduino_pin arduino_in_pins[NR_OF_IN_PINS];

void digin_callback(uint8_t pin, uint8_t val); 

void 
digin_callback(uint8_t pin, uint8_t val)
{
  PRINT_FUNCTION_NAME(("%d,%d",pin,val));

  if (pin<0 || pin> NR_OF_IN_PINS)
    {
      SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
      return 0;
    }

  arduino_in_pins[pin].val=val;
  printf ("       - %d %d (%d)\n", pin, arduino_in_pins[pin].val, val);
}

static void searduinio_setup(void)
{
  static int already_setup = 0;
  int ret;

  if (already_setup)
    {
      return;
    }
  PRINT_FUNCTION_NAME((""));
  comm_setup();
  ret = comm_register_digin_cb(digin_callback);

  if (ret != SEARD_PROTO_OK)
    {
      fprintf(stderr, "Failed to register callback");
    }

  already_setup=1;
}



void pinMode(uint8_t pin, uint8_t mode)
{
  searduinio_setup();
  PRINT_FUNCTION_NAME(("%d,%d",pin,mode));
  if (pin<0 || pin> NR_OF_OUT_PINS)
    {
      SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
      return;
    }
  arduino_out_pins[pin].mode=mode;

  return;
}

static void turnOffPWM(uint8_t timer)
{
  PRINT_FUNCTION_NAME(("%d",timer));
}

void digitalWrite(uint8_t pin, uint8_t val)
{
  int ret;
  searduinio_setup();

  PRINT_FUNCTION_NAME(("%d,%d",pin,val));
  if (pin<0 || pin> NR_OF_OUT_PINS)
    {
      SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
      return;
    }

  if ( arduino_out_pins[pin].mode== 0 )
    {
      fprintf(stderr, "Can't write to a pin with mode set 0\n");
      SEARD_ERROR(SEARD_ARDUINO_PIN_MODE_NOT_SET);
      return;
    }
  arduino_out_pins[pin].val=val;

  ret = comm_digital_write(pin,val);
  if (ret != SEARD_ARDUINO_OK)
    {
      SEARD_ERROR(ret);
      return;
    }
  
  return;
}

int digitalRead(uint8_t pin)
{
  if (pin<0 || pin> NR_OF_IN_PINS)
    {
      SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
      return 0;
    }

  return arduino_in_pins[pin].val;
}

